<?php

namespace WPSocialReviews\Classes;

use WPSocialReviews\Classes\Helper;
use WPSocialReviews\Classes\Models\Templates;
use WPSocialReviews\Classes\Reviews\GoogleBusiness;

// use WPSocialReviews\Classes\Reviews\GoogleReview;
use WPSocialReviews\Classes\Models\ReviewsModel;

if (!defined('ABSPATH')) {
    exit;
}

class AdminAjaxHandler
{
    /**
     *
     * Register end points
     *
     * @since 1.0.0
     *
     **/
    public function registerEndpoints()
    {
        add_action('wp_ajax_wpsocial_review_admin_ajax', array($this, 'handeEndPoint'));

    }

    /**
     *
     * validate route
     *
     * @return method name
     * @since 1.0.0
     *
     **/
    public function handeEndPoint()
    {
        $route = sanitize_text_field($_REQUEST['route']);
        $validRoutes = array(

            //api verification related routes
            'save_api_settings' => 'handleCredentialSave',
            'get_api_key' => 'getCredentials',
            'clear_verification_credentials' => 'clearVerificationCredentials',

            'add_new_template' => 'addNewTemplate',

            'get_valid_platforms' => 'getValidPlatforms',

            'save_template_meta' => 'saveTemplateMeta',
            'get_template_meta' => 'getTemplateMeta',

            'get_all_template' => 'getAllTemplate',
            'delete_template' => 'deleteTemplate',
            'duplicate_template' => 'duplicateTemplate',

            'save_template_title' => 'saveTemplateTitle',

            'get_user_data' => 'getUserData',

            'get_update_template_meta_and_reviews' => 'getUpdateTemplateMetaAndReviews',

            //custom review
            'get_all_reviews' => 'getAllReviews',
            'save_custom_review' => 'saveCustomReview',
            'wpsr_upload_image' => 'handleFileUpload',
            'get_review_by_id' => 'getReviewById',
            'update_review' => 'updateReview',
            'delete_review' => 'deleteReview',
            'duplicate_review' => 'duplicateReview',

            //advance settings
            'get_advance_settings' => 'getAdvanceSettings',
            'save_advance_settings' => 'saveAdvanceSettings',

        );

        if (isset($validRoutes[$route])) {
            do_action('wpsocialreviews/doing_ajax_action_' . $route);
            return $this->{$validRoutes[$route]}();
        }
        do_action('wpsocialreviews/admin_ajax_handler_catch', $route);
    }

    /**
     *
     * Get a valid platform name from wp social reviews table
     *
     * @return string
     * @since 1.0.0
     *
     **/
    public function getValidPlatforms()
    {
        $platforms = apply_filters('wpsocialreviews/available_valid_platforms', []);
        //add custom with platforms if custom reviews exists
        $isCustomReviewsExists = ReviewsModel::isCustomFound();
        if ($isCustomReviewsExists) {
            $platforms['custom'] = __('Custom', 'wpsocialreviews');
        }
        Helper::sendSuccess([
            'message' => 'success',
            'platforms' => $platforms
        ]);
    }

    /**
     *
     * Add a new template on wp posts table
     *
     * @return string
     * @since 1.0.0
     *
     **/
    public function addNewTemplate()
    {
        $platform = sanitize_text_field($_REQUEST['platform']);

        $platform = $platform == 'google_my_business' ? 'google' : $platform;

        $templateData = array(
            'post_title' => $platform . ' template',
            'post_content' => 'empty_' . $platform,
            'post_type' => PostType::$CPTName,
            'post_status' => 'publish'
        );

        $templateId = wp_insert_post($templateData);

        do_action('wpsocialreviews/before_create_template', $platform, $templateData);

        wp_update_post([
            'ID' => $templateId,
            'post_title' => $platform . ' template' . ' (#' . $templateId . ')'
        ]);

        if (is_wp_error($templateId)) {
            wp_send_json_error(array(
                'message' => $templateId->get_error_message()
            ), 423);
        }

        do_action('wpsocialreviews/after_create_template', $templateId, $platform, $templateData);

        wp_send_json_success(array(
            'message' => __('Template Successfully created', 'wpsocialreviews'),
            'template_id' => $templateId
        ), 200);
    }

    public function getUpdateTemplateMetaAndReviews()
    {
        $args = json_decode(wp_unslash($_REQUEST['template_meta']));
        $args = json_decode(json_encode($args), true);
        $templateId = intval($_REQUEST['template_id']);

        $templateDetails = get_post($templateId);
        $reviewsData = $this->collectReviewsAndBusinessInfo($args);

        Helper::sendSuccess([
            'message' => 'success',
            'template_id' => $templateId,
            'filtered_reviews' => $reviewsData['filtered_reviews'],
            'all_reviews' => $reviewsData['all_reviews'],
            'shop_details' => $reviewsData['business_info'],
            'template_details' => $templateDetails,
            'template_meta' => $args
        ]);
    }

    public function collectReviewsAndBusinessInfo($settings = array())
    {
        $platforms = isset($settings['platform']) ? $settings['platform'] : array();
        $activePlatforms = apply_filters('wpsocialreviews/available_valid_platforms', []);
        if (empty($platforms)) {
            $platforms = array();
        }
        $validTemplatePlatforms = array_intersect($platforms, array_keys($activePlatforms));

        //add custom with platforms if custom reviews exists
        $isCustomReviewsExists = ReviewsModel::isCustomFound();
        if ($isCustomReviewsExists && in_array("custom", $platforms)) {
            $index = array_search('custom', $platforms);
            $validTemplatePlatforms[$index] = 'custom';
        }

        $filtered_reviews = array();
        $all_reviews = array();
        $business_info = array();

        if (!empty($validTemplatePlatforms)) {
            $key = array_keys($validTemplatePlatforms)[0];

            $filtered_reviews = ReviewsModel::getFilteredReviewsByPlatforms($validTemplatePlatforms, $settings);

            $businessInfo = array();
            if (isset($validTemplatePlatforms[$key])) {
                $businessInfo = get_option('social_review_' . $validTemplatePlatforms[$key] . '_business_info');
            }

            $multi_business_info = Helper::getMultiBusinessInfo($validTemplatePlatforms);
            $business_info = sizeof($validTemplatePlatforms) > 1 ? $multi_business_info : $businessInfo;
            $all_reviews = $this->getReviewsByPlatforms($validTemplatePlatforms);
        }

        $data = array(
            'filtered_reviews' => $filtered_reviews,
            'all_reviews' => $all_reviews,
            'business_info' => $business_info
        );

        return $data;
    }

    public function getReviewsByPlatforms($platforms)
    {
        $reviews = wpSocialReviewsDB()
            ->table('wpsr_reviews')
            ->whereIn('platform_name', $platforms)
            ->get();

        return $reviews;
    }

    /**
     *
     * Save api key
     *
     * @throws /Exception
     * @since 1.0.0
     *
     **/
    public function handleCredentialSave()
    {
        $platform = sanitize_text_field($_REQUEST['platform']);
        delete_option('social_review_' . $platform . '_business_info');
        do_action('wpsocialreviews/verify_review_credential_' . $platform);
    }

    /**
     *
     * Ger API key
     *
     * @throws /Exception
     * @since 1.0.0
     *
     **/
    public function getCredentials()
    {
        $platform = sanitize_text_field($_REQUEST['platform']);
        $credential = apply_filters('wpsocialreviews/api_credential_' . $platform, []);
        $businessInfo = apply_filters('wpsocialreviews/business_info_' . $platform, [
            'name' => '',
            'url' => '',
            'address' => '',
            'average_rating' => '',
            'total_rating' => '',
            'phone' => '',
            'platform_name' => '',
            'status' => false
        ]);

        $additionalInfo = apply_filters('wpsocialreviews/additional_info_' . $platform, []);

        wp_send_json_success([
            'credential' => $credential,
            'business_info' => $businessInfo,
            'additional_info' => $additionalInfo,
            //'advance_additional_info' =>  get_option('google_my_business_advance_settings',false)
        ], 200);
    }

    /**
     *
     * Remove api key and place id from wp options table
     *
     * @throws /Exception
     * @since 1.2.1
     *
     **/
    public function clearVerificationCredentials()
    {
        $platform = sanitize_text_field($_REQUEST['platform']);
        do_action('wpsocialreviews/clear_review_credential_' . $platform);
    }

    /**
     *
     * Save template meta
     *
     * @throws /Exception
     * @since 1.0.0
     *
     **/
    public function saveTemplateMeta()
    {
        $args = json_decode(wp_unslash($_REQUEST['template_meta']));
        $args = json_decode(json_encode($args), true);
        $templateId = intval($_REQUEST['template_id']);
        self::updateTemplateMeta($templateId, $args);
    }

    /**
     *
     * Update template meta by template id
     *
     * @param $templateId
     * @param $templateMetaData
     * @throws /Exception
     * @since 1.0.0
     *
     **/
    public static function updateTemplateMeta($templateId, $templateMetaData)
    {
        do_action('wpsocialreviews/template_meta_data', $templateId, $templateMetaData);
        update_post_meta($templateId, '_wpsr_template_config', $templateMetaData);
        $templateConfig = get_post_meta($templateId, '_wpsr_template_config', true);
        Helper::sendSuccess([
            'message' => 'success',
            'template_id' => $templateId,
            'template_meta' => $templateConfig,
        ]);
    }

    /**
     *
     * Get template meta
     *
     * @throws /Exception
     * @since 1.0.0
     *
     **/
    public function getTemplateMeta()
    {
        $templateId = intval($_REQUEST['template_id']);
        $templateDetails = get_post($templateId);
        $templateConfig = get_post_meta($templateId, '_wpsr_template_config', true);

        $settings = $templateConfig ? $templateConfig : array();
        $formattedConfig = ReviewsModel::formattedTemplateMeta($settings);

        $reviewsData = $this->collectReviewsAndBusinessInfo($formattedConfig);

        Helper::sendSuccess([
            'message' => 'success',
            'template_id' => $templateId,
            'filtered_reviews' => $reviewsData['filtered_reviews'],
            'all_reviews' => $reviewsData['all_reviews'],
            'shop_details' => $reviewsData['business_info'],
            'template_details' => $templateDetails,
            'template_meta' => $formattedConfig
        ]);
    }

    /**
     *
     * Get all template
     *
     * @return string
     * @throws /Exception
     * @since 1.0.0
     *
     **/
    public function getAllTemplate()
    {
        $perPage = absint($_REQUEST['per_page']);
        $pageNumber = absint($_REQUEST['page_number']);
        $args = array(
            'posts_per_page' => $perPage,
            'offset' => $perPage * ($pageNumber - 1)
        );

        apply_filters('wpsocialreviews/get_all_template_args', $args);

        $searchString = sanitize_text_field($_REQUEST['search_string']);
        if ($searchString) {
            $args['s'] = $searchString;
        }

        $allData = Templates::getTemplates($args, $with = array('platform_name'));
        Helper::sendSuccess([
            'message' => 'success',
            'allTemplateData' => $allData,
        ]);
    }

    /**
     *
     * Delete template
     *
     * @throws /Exception
     * @since 1.0.0
     *
     **/
    public function deleteTemplate()
    {
        $templateId = intval($_REQUEST['template_id']);
        do_action('wpsocialreviews/before_template_delete', $templateId);
        Templates::deleteTemplate($templateId);
        do_action('wpsocialreviews/after_template_delete', $templateId);
        Helper::sendSuccess([
            'message' => __('Template deleted successfully', 'wpsocialreviews'),
        ]);
    }

    /**
     *
     * Save template title
     *
     * @throws /Exception
     * @since 1.0.0
     *
     **/
    public function saveTemplateTitle()
    {
        $templateTitle = sanitize_text_field($_REQUEST['template_title']);
        $templateId = intval($_REQUEST['template_id']);

        do_action('wpsocialreviews/before_save_title', $templateId);
        $res = Templates::saveTitle($templateId, $templateTitle);
        do_action('wpsocialreviews/after_save_title', $templateId);

        if ($res) {
            Helper::sendSuccess([
                'message' => 'success',
                'title' => $templateTitle,
                'id' => $templateId,
                'result' => $res
            ]);
        } else {
            wp_send_json_error(array(
                'message' => "no success",
            ), 423);
        }
    }

    public function duplicateTemplate()
    {
        $templateId = absint($_POST['template_id']);
        $oldTemplate = Templates::getTemplate($templateId);
        $oldTemplate['post_title'] = '(Duplicate) ' . $oldTemplate['post_title'];
        $oldTemplate = apply_filters('wpsocialreviews/template_duplicate', $oldTemplate);

        if (!$oldTemplate) {
            wp_send_json_error(array(
                'message' => __('No template found when duplicating the template', 'wpsocialreviews')
            ), 423);
        }
        $newTemplate = Templates::createNewTemplateData($oldTemplate);
        wp_send_json_success(array(
            'message' => __('Template successfully duplicated', 'wpsocialreviews'),
            'template' => $newTemplate,
        ), 200);
    }

    public function getUserData()
    {
        $platform = sanitize_text_field($_REQUEST['platform']);
        do_action('wpsocialreviews/get_user_data' . $platform);
    }

    public function getAdvanceSettings()
    {
        $platform = sanitize_text_field($_REQUEST['platform']);
        do_action('wpsocialreviews/get_advance_settings_' . $platform);
    }

    public function saveAdvanceSettings()
    {
        $platform = sanitize_text_field($_REQUEST['platform']);
        do_action('wpsocialreviews/save_advance_settings_' . $platform);
    }

    public function saveCustomReview()
    {
        $customReview = wp_unslash($_REQUEST['custom_review']);

        if (empty($customReview['reviewer_name']) ||
            empty($customReview['reviewer_text']) || empty($customReview['reviewer_rating']) ||
            empty($customReview['review_time']) || empty($customReview['review_platform'])) {

            $this->validateField($customReview);

        } else {

            $newReview = $this->formatData($customReview);

            wpSocialReviewsDB()->table('wpsr_reviews')
                ->insert($newReview);

            Helper::sendSuccess([
                'message' => 'custom review saved!',
                'new review' => $newReview,
            ]);

        }
    }

    public function formatData($review)
    {
        return [
            'platform_name' => $review['review_platform'],
            'source_id' => 'custom',
            'reviewer_name' => $review['reviewer_name'],
            'review_title' => 'custom_' . time(),
            'reviewer_url' => $review['reviewer_url'],
            'reviewer_img' => $review['reviewer_image'],
            'reviewer_text' => $review['reviewer_text'],
            'rating' => $review['reviewer_rating'],
            'review_time' => $review['review_time'],
            'recommendation_type' => 'positive',
            'updated_at' => date('Y-m-d H:i:s'),
            'created_at' => date('Y-m-d H:i:s')
        ];
    }

    public function handleFileUpload()
    {
        if (!function_exists('wp_handle_upload')) {
            require_once(ABSPATH . 'wp-admin/includes/file.php');
        }
        $uploadedfile = $_FILES['file'];

        $acceptedFilles = array(
            'image/png',
            'image/jpeg'
        );

        if (!in_array($uploadedfile['type'], $acceptedFilles)) {
            wp_send_json(__('Please upload only jpg/png format files', 'wpsocialreviews'), 423);
        }

        $upload_overrides = array('test_form' => false);

        $movefile = wp_handle_upload($uploadedfile, $upload_overrides);

        if ($movefile && !isset($movefile['error'])) {
            wp_send_json_success(array(
                'file' => $movefile
            ), 200);
        } else {
            wp_send_json(__('Something is wrong when uploading the file', 'wpsocialreviews'), 423);
        }
    }

    public function getAllReviews()
    {
        $perPage = absint($_REQUEST['per_page']);
        $pageNumber = absint($_REQUEST['page_number']);
        $args = array(
            'posts_per_page' => $perPage,
            'offset' => $perPage * ($pageNumber - 1)
        );
        $searchString = sanitize_text_field($_REQUEST['search_string']);
        if ($searchString) {
            $args['s'] = $searchString;
        }
        $allReviews = ReviewsModel::getAllQueryReviews($args);
        Helper::sendSuccess([
            'message' => 'success',
            'allReviews' => $allReviews,
        ]);
    }

    public function getReviewById()
    {
        $reviewId = absint($_REQUEST['review_id']);
        $review = ReviewsModel::getReviewById($reviewId);
        Helper::sendSuccess([
            'message' => 'success',
            'review' => $review,
        ]);
    }

    public function updateReview()
    {
        $reviewData = wp_unslash($_REQUEST['review_data']);
        $reviewId = absint($_REQUEST['review_id']);
        if (empty($reviewData['reviewer_name']) || empty($reviewData['reviewer_text']) ||
            empty($reviewData['reviewer_rating']) || empty($reviewData['review_time']) ||
            empty($reviewData['review_platform'])) {

            $this->validateField($reviewData);

        } else {
            ReviewsModel::updateReview($reviewId, $reviewData);
            Helper::sendSuccess([
                'message' => 'success',
                'review' => $reviewData,
            ]);
        }
    }

    public function deleteReview()
    {
        $reviewData = wp_unslash($_REQUEST['review_data']);
        $review = ReviewsModel::deleteReview($reviewData);
        Helper::sendSuccess([
            'message' => __('Successfully Deleted!', 'wpsocialreviews'),
            'review' => $review,
        ]);
    }

    public function validateField($review)
    {
        $errors = array();

        if (empty($review['reviewer_name'])) {
            $errors['reviewer_name'] = __('Reviewer Name is required!', 'wpsocialreviews');
        }

        if (empty($review['reviewer_text'])) {
            $errors['reviewer_text'] = __('Reviewer Text is required!', 'wpsocialreviews');
        }

        if (empty($review['reviewer_rating'])) {
            $errors['reviewer_rating'] = __('Reviewer Rating is required!', 'wpsocialreviews');
        }

        if (empty($review['review_time'])) {
            $errors['review_time'] = __('Reviewer Time is required!', 'wpsocialreviews');
        }

        if (empty($review['review_platform'])) {
            $errors['review_platform'] = __('Review Platform is required!', 'wpsocialreviews');
        }

        wp_send_json_error([
            'message' => 'error',
            'errors' => $errors,
        ], 423);
    }

    public function duplicateReview()
    {
        $review = wp_unslash($_REQUEST['review']);
        $reviewData = ReviewsModel::duplicateReview($review);
        Helper::sendSuccess([
            'message' => __('Successfully duplicated!', 'wpsocialreviews'),
            'new_review' => $reviewData,
            'old_review' => $review,
        ]);
    }

}
