<?php

namespace WPSocialReviews\Classes\Feeds\Youtube;

use WPSocialReviews\Classes\Helper;
use WPSocialReviews\Classes\Feeds\BaseFeed;
use WPSocialReviews\Classes\Feeds\CacheHandler;
use WPSocialReviews\Classes\Feeds\Common\FeedFilters;

if (!defined('ABSPATH')) {
    exit;
}

class YoutubeFeed extends BaseFeed
{

    private static $totalFeed = 10;
    private $totalComment = 10;
    protected $oauth;
    protected $cacheHandler;
    protected $errorCode = false;
    private $remoteFetchUrl = 'https://www.googleapis.com/youtube/v3/';
    private $comments = array();

    public function __construct()
    {
        parent::__construct('youtube');
        $this->oauth = new OAuth();
        $this->cacheHandler = new CacheHandler('youtube');
    }

    /**
     * Handle YouTube Feed
     *
     * Make sure all data related to you tube settings are formatted with dynamic youtube feed data from api
     *
     * @return array
     * @since 1.2.5
     */
    public function getFeedSettings()
    {
        $postId = intval($_REQUEST['post_id']);
        $feed_template_meta = get_post_meta($postId, '_wpsr_youtube_template_config', true);

        $configs = isset($feed_template_meta['feed_settings']) ? $feed_template_meta['feed_settings'] : array();

        $formatted_feed_template_meta = self::formatYoutubeConfig($configs, array());

        //feed settings to simplify repetitive use
        $feed_settings = isset($formatted_feed_template_meta['feed_settings']) ? $formatted_feed_template_meta['feed_settings'] : array();

        //filter the feeds
        $apiSettings = isset($feed_settings['source_settings']) ? $feed_settings['source_settings'] : array();
        $response = $this->apiConnection($apiSettings, $postId);

        $filterSettings = isset($feed_settings['filters']) ? $feed_settings['filters'] : array();
        $filterResponse = array();

        if (isset($response['error_message'])) {
            $filterResponse = $response;
        } else {
            $filterResponse = (new FeedFilters)->filterFeedResponse($this->platform, $filterSettings, $response);
        }

        $templateDetails = get_post($postId);
        $formatted_feed_template_meta['dynamic'] = $filterResponse;
        $configs = $formatted_feed_template_meta;

        Helper::sendSuccess([
            'message' => __('Success', 'wpsocialreviews'),
            'settings' => $configs,
            'template_details' => $templateDetails,
            'errorCode' => $this->errorCode

        ]);
    }

    /**
     * Handle Api key or token
     **
     * @return string
     * @since 1.2.5
     */
    protected function getApiKeyOrToken()
    {
        $apiCredentials = get_option('wpsr_youtube_verification_configs', true);
        $youtubeApiKeyOrToken = '';
        $youtubeAccessToken = '';
        $youtubeApiKey = '';


        if (isset($apiCredentials['credentials_type']) && $apiCredentials['credentials_type'] === 'oauth2.0') {

            $youtubeAccessToken = $this->oauth->getAccessToken();

            if (!$youtubeAccessToken) {

                $message = array(
                    'error_message' => __('Error in api connection,Please configure your youtube settings again.!! ', 'wpsocialreviews')
                );

                return $message;
            }

        } elseif (isset($apiCredentials['credentials_type']) && $apiCredentials['credentials_type'] === 'api_key') {
            $youtubeApiKey = $apiCredentials['api_key'];
        } elseif (isset($apiCredentials['credentials_type']) && $apiCredentials['credentials_type'] === 'manually_connect') {
            $youtubeAccessToken = $apiCredentials['access_token'];
        }

        if (!empty($youtubeAccessToken) && empty($youtubeApiKey)) {
            $youtubeApiKeyOrToken = 'access_token=' . $youtubeAccessToken . '';
        } else {
            $youtubeApiKeyOrToken = 'key=' . $youtubeApiKey . '';
        }

        if (!$youtubeAccessToken && !$youtubeApiKey) {

            $message = array(
                'error_message' => __('No credential found. Please configure your youtube settings correctly', 'wpsocialreviews')
            );

            return $message;
        }

        return $youtubeApiKeyOrToken;
    }

    /**
     * Handle YouTube Api Connection
     *
     * Collect youtube data from api
     * @param $apiSettings
     * @param $postId
     * @return array
     * @since 1.2.5
     */
    public function apiConnection($apiSettings, $postId = null)
    {
        $total = isset($apiSettings['feed_count']) ? intval($apiSettings['feed_count']) : 10;
        $feedType = isset($apiSettings['feed_type']) ? $apiSettings['feed_type'] : 'channel_feed';

        $channelId = isset($apiSettings['channel_id']) ? $apiSettings['channel_id'] : '';
        $playlistId = isset($apiSettings['playlist_id']) ? $apiSettings['playlist_id'] : '';
        $user_name = isset($apiSettings['user_name']) ? $apiSettings['user_name'] : '';
        $searchTerm = isset($apiSettings['search_term']) ? $apiSettings['search_term'] : '';
        $videoIds = isset($apiSettings['video_id']) ? $apiSettings['video_id'] : '';
        $eventType = isset($apiSettings['event_type']) ? $apiSettings['event_type'] : '';
        if (empty($channelId) && empty($playlistId) && empty($searchTerm) && empty($videoIds)) {
            $message = array(
                'error_message' => __('Please enter channel id, playlist id, search term or single video id to fetch videos!! ', 'wpsocialreviews')
            );
            return $message;
        }

        $apiDetails = array();

        if (!defined('WPSOCIALREVIEWS_PRO') && $feedType !== 'channel_feed') {
            return array('error_message' => __('Please upgrade to pro version to use this features!! ', 'wpsocialreviews'));
        }

        if ($feedType === 'channel_feed') {
            $apiDetails = $this->getChannelApiUrlDetails($channelId, $total);
        } else if ($feedType === 'playlist_feed') {
            $apiDetails = apply_filters('wpsocialreviews/youtube_playlist_api_url_details', $playlistId, $total, $this->remoteFetchUrl);
        } else if ($feedType === 'search_feed') {
            $apiDetails = apply_filters('wpsocialreviews/youtube_search_api_url_details', $searchTerm, $total, $this->remoteFetchUrl);
        } else if ($feedType === 'live_streams') {
            $apiDetails = apply_filters('wpsocialreviews/youtube_live_streams_api_url_details', $channelId, $eventType, $total, $this->remoteFetchUrl);
        } else if ($feedType === 'single_video') {
            $videoIds = array_map('trim', explode(",", $videoIds));
            $feedCacheName = 'wpsr_youtube_videos_list_' . $postId . '_total_' . count($videoIds);
            return $this->getSingleVideoFeeds($videoIds, $feedCacheName);
        }

        if (isset($apiDetails['error_message'])) {
            return array('error_message' => $apiDetails['error_message']);
        }

        $youtubeFeedApiUrl = $apiDetails['api_url'];
        $feedCacheName = $apiDetails['cache_name'];

        //without single video feed we have now api url for fetching videos
        return $this->getYoutubeFeed($feedType, $youtubeFeedApiUrl, $feedCacheName, $channelId, $total);
    }

    public function getVideoIds($feeds = array())
    {
        $videoIds = array();
        if (isset($feeds['items']) && count($feeds['items'])) {
            foreach ($feeds['items'] as $index => $feed) {
                $videoId = isset($feed['snippet']['resourceId']['videoId']) ? $feed['snippet']['resourceId']['videoId'] : '';
                if (empty($videoId)) {
                    $videoId = isset($feed['id']['videoId']) ? $feed['id']['videoId'] : '';
                }
                if (empty($videoId)) {
                    $videoId = isset($feed['id']) ? $feed['id'] : '';
                }
                $videoIds[] = $videoId;
            }
        }
        return $videoIds;
    }

    /**
     * Youtube Feeds Without Single Video
     *
     * @string $feedType feed type
     * @string $youtubeFeedApiUrl feed api url
     * @string $feedCacheName cache name
     * @string  $channelId channel id
     * @integer $total total
     * @return array
     * @throws /Exception
     * @since 1.2.5
     */
    public function getYoutubeFeed($feedType, $youtubeFeedApiUrl, $feedCacheName, $channelId, $total)
    {

        $channelInfo = '';
        if ($youtubeFeedApiUrl) {

            $videoLists = array();
            if (true === $this->cacheHandler->checkFeedCacheExists($feedCacheName)) {

                // Return Cache because it exists in Database.
                $videoLists = $this->cacheHandler->getFeedCache($feedCacheName);

            } else {

                $youtubeApiKeyOrToken = $this->getApiKeyOrToken();

                if (isset($youtubeApiKeyOrToken['error_message'])) {
                    $message = array(
                        'error_message' => $youtubeApiKeyOrToken
                    );
                    return $message;
                }

                $feeds = $this->getAPIData($youtubeFeedApiUrl . $youtubeApiKeyOrToken);

                if (isset($feeds['error']['message'])) {
                    $error = $feeds['error']['message'];
                    $message = array('error_message' => $error);
                    return $message;
                }

                //we are storing video ids for using in videos api with comma separator
                $videoIds = $this->getVideoIds($feeds);

                $this->comments = apply_filters('wpsocialreviews/youtube_single_video_comments', $videoIds, $feedCacheName, $youtubeApiKeyOrToken, $this);

                $videoIds = implode(',', $videoIds);

                $parts = 'id,snippet';
                $parts = apply_filters('wpsocialreviews/youtube_api_parts', $parts, $feedType);

                $channelVideoApiUrl = $this->remoteFetchUrl . 'videos?part=' . $parts . '&id=' . $videoIds . '&';
                $videoLists = $this->checkYoutubeCache($channelVideoApiUrl, $feedCacheName);

            }

            if ($feedType === 'playlist_feed' && isset($videoLists['items']) && count($videoLists['items'])) {

                foreach ($videoLists['items'] as $video) {

                    if (isset($video['snippet']['channelId'])) {

                        $channelId = $video['snippet']['channelId'];
                        break;

                    }

                }

            }

            $feedTypes = array('playlist_feed', 'channel_feed', 'live_streams');
            if (!empty($channelId) && in_array($feedType, $feedTypes)) {

                $channelApiParams = 'id=' . $channelId;
                $channelCacheName = 'wpsr_youtube_channel_header_' . $channelId . '_total_' . $total;

                $youtubeChannelApiUrl = $this->remoteFetchUrl . 'channels?part=id,snippet,contentDetails,statistics,brandingSettings&maxResults=' . $total . '&' . $channelApiParams . '&';
                $videos = $this->checkYoutubeCache($youtubeChannelApiUrl, $channelCacheName);
                $channelInfo = $videos;

            }

            $videoLists['channel_info'] = $channelInfo;
            return $videoLists;

        }
    }

    /**
     * Youtube Single Video Feeds
     *
     * @param array $videoIds video ids
     * @return array
     * @throws /Exception
     * @since 1.2.5
     */
    public function getSingleVideoFeeds($videoIds, $feedCacheName)
    {
        if (empty($videoIds)) {
            $data = array(
                'error_message' => __('Please enter a video id!! ', 'wpsocialreviews')
            );
            return $data;
        }

        $videoLists = array();
        if (!empty($videoIds)) {

            $parts = 'id,snippet';
            $parts = apply_filters('wpsocialreviews/youtube_api_parts', $parts, 'single_video');

            if (true === $this->cacheHandler->checkFeedCacheExists($feedCacheName)) {
                $videoLists = $this->cacheHandler->getFeedCache($feedCacheName);
            } else {

                $youtubeApiKeyOrToken = $this->getApiKeyOrToken();

                if (isset($youtubeApiKeyOrToken['error_message'])) {
                    $message = array(
                        'error_message' => $youtubeApiKeyOrToken
                    );
                    return $message;
                }

                $this->comments = apply_filters('wpsocialreviews/youtube_single_video_comments', $videoIds, $feedCacheName, $youtubeApiKeyOrToken, $this);

                $videoIds = implode(',', $videoIds);
                $channelVideoApiUrl = $this->remoteFetchUrl . 'videos?part=' . $parts . '&id=' . $videoIds . '&';
                $videoLists = $this->checkYoutubeCache($channelVideoApiUrl, $feedCacheName);
            }

        }

        $videoLists['channel_info'] = '';
        return $videoLists;
    }

    /**
     * Youtube Channel Api Url Details
     *
     * @param string $channelId Channel ID
     * @param integer $total total
     * @return array
     * @throws /Exception
     * @since 1.2.5
     */
    public function getChannelApiUrlDetails($channelId, $total)
    {
        $feedCacheName = '';
        $youtubeFeedApiUrl = '';
        if (empty($channelId)) {

            $data = array(
                'error_message' => __('Please enter channel id to fetch videos!! ', 'wpsocialreviews')
            );
            return $data;

        }
        $channelApiParams = 'mine=true';
        if ($channelId) {
            $channelApiParams = 'id=' . $channelId;
        }
        // elseif ( $playlistId  ) {
        // 	$channelApiParams = 'forUsername=' . $playlistId;
        // }
        $channelCacheName = 'wpsr_youtube_channel_header_' . $channelId . '_total_' . $total;
        $youtubeChannelApiUrl = $this->remoteFetchUrl . 'channels?part=id,snippet,contentDetails,statistics,brandingSettings&maxResults=' . $total . '&' . $channelApiParams . '&';
        $videos = $this->checkYoutubeCache($youtubeChannelApiUrl, $channelCacheName);

        $resultSet = isset($videos['pageInfo']['resultsPerPage']) ? $videos['pageInfo']['resultsPerPage'] : 0;
        if (!$resultSet) {
            $data = array(
                'error_message' => __('Can\'t find any videos to the channel!!', 'wpsocialreviews')
            );
            return $data;
        }
        $playlist = isset($videos['items'][0]['contentDetails']['relatedPlaylists']['uploads']) ? $videos['items'][0]['contentDetails']['relatedPlaylists']['uploads'] : '';
        if ($playlist) {
            $feedCacheName = 'wpsr_youtube_videos_list_' . $playlist . '_total_' . $total;
            $youtubeFeedApiUrl = $this->remoteFetchUrl . 'playlistItems?part=id,snippet&maxResults=' . $total . '&playlistId=' . $playlist . '&';
        }
        $data = array(
            'cache_name' => $feedCacheName,
            'api_url' => $youtubeFeedApiUrl
        );
        return $data;
    }

    /**
     * Handle YouTube Cache
     *
     * @param string $youtubeUrl youtube url
     * @param string $cacheName cache name
     * @return array|string
     * @throws /Exception
     * @since 1.2.5
     */
    public function checkYoutubeCache($youtubeUrl, $cacheName)
    {
        if (true === $this->cacheHandler->checkFeedCacheExists($cacheName)) {
            // Return Cache because it exists in Database.
            return $this->cacheHandler->getFeedCache($cacheName);
        }
        $youtubeApiKeyOrToken = $this->getApiKeyOrToken();

        if (isset($youtubeApiKeyOrToken['error_message'])) {
            if (true === $this->cacheHandler->checkFeedCacheExists($cacheName, true)) {
                // Return Cache because it exists in Database. Better than showing nothing right?
                return $this->cacheHandler->getFeedCache($cacheName, true);
            } else {
                $message = array(
                    'error_message' => $youtubeApiKeyOrToken['error_message']
                );
                return $message;
            }
        }

        $response = wp_remote_get($youtubeUrl . $youtubeApiKeyOrToken);
        $feedData = json_decode(wp_remote_retrieve_body($response), true);

        if (isset($feedData['items']) && strpos($cacheName, 'channel') === false && strpos($youtubeApiKeyOrToken, 'key') !== false) {
            foreach ($feedData['items'] as $index => $feed) {

                $videoId = isset($feed['snippet']['resourceId']['videoId']) ? $feed['snippet']['resourceId']['videoId'] : '';
                if (empty($videoId)) {
                    $videoId = isset($feed['id']['videoId']) ? $feed['id']['videoId'] : '';
                }
                if (empty($videoId)) {
                    $videoId = isset($feed['id']) ? $feed['id'] : '';
                }

                $feedData['items'][$index]['comment'] = isset($this->comments[$videoId]) ? $this->comments[$videoId] : '';

            }
        }

        if (!empty($feedData) && !empty($cacheName)) {
            $wpsr_error_check_complete = $this->checkYoutubeError($feedData);
            if (is_array($wpsr_error_check_complete) && (true === $wpsr_error_check_complete[0] || 1 === $wpsr_error_check_complete[0])) {
                // If old Cache still exists use it instead of showing an error. This prevents feed from breaking if old cache is still there.
                if (true === $this->cacheHandler->checkFeedCacheExists($cacheName, true)) {
                    // Return Cache because it exists in Database. Better than showing nothing right?
                    return $this->cacheHandler->getFeedCache($cacheName, true);
                }
                if (strpos($cacheName, 'wpsr_video_single_comments') !== false) {
                    $this->cacheHandler->createFeedCache($cacheName, array());
                }
                $message = array(
                    'error_message' => isset($wpsr_error_check_complete[1]) ? $wpsr_error_check_complete[1] : __('Problem in api key or access token setting l!!', 'wpsocialreviews')
                );
                return $message;
            } elseif (!$this->cacheHandler->checkFeedCacheExists($cacheName)) {
                // Create Cache.
                $this->cacheHandler->createFeedCache($cacheName, $feedData);
                return $feedData;
            }
        } else if (!empty($cacheName) && empty($feedData)) {
            // If old Cache still exists use it instead of showing an error. This prevents feed from breaking if old cache is still there.
            if (true === $this->cacheHandler->checkFeedCacheExists($cacheName, true)) {
                // Return Cache because it exists in Database. Better than showing nothing right?
                return $this->cacheHandler->getFeedCache($cacheName, true);
            }
        }
    }

    /**
     * Handle Error
     *
     * Check if there any error in youtube api data
     *
     * @param array $feedData feed data
     * @return array
     * @throws /Exception
     * @since 1.2.5
     */
    public function checkYoutubeError($feedData)
    {
        try {
            if (isset($feedData['error']) && 400 === $feedData['error']['code']) {
                $this->errorCode = $feedData['error']['code'];
                throw new \Exception($feedData['error']['code'] . ' - A VALID access token is required to request this resource.');
            }
            if (isset($feedData['error'])) {
                // If Custom Exception is not needed but still error then throw ugly error.
                if (isset($feedData['error']['message'])) {
                    $output = 'Error: ' . $feedData['error']['message'];
                }
                if (isset($feedData['error']['code'])) {
                    $this->errorCode = $feedData['error']['code'];
                    $output .= '<br />Code: ' . $feedData['error']['code'];
                }
                throw new \Exception($output);
            }

        } catch (\Exception $e) {
            return array(true, $e->getMessage());
        }
        return;
    }

    /**
     * Format youtube settings
     *
     * @param array $settings settings
     * @param array $response response
     * @return array
     * @since 1.2.5
     */
    public static function formatYoutubeConfig($settings, $response)
    {
        return array(

            'feed_settings' => array(

                'platform' => 'youtube',
                'template' => isset($settings['template']) ? $settings['template'] : 'template1',
                'layout_type' => isset($settings['layout_type']) ? $settings['layout_type'] : 'grid',
                'column_number' => isset($settings['column_number']) ? $settings['column_number'] : '4',
                'column_gaps' => isset($settings['column_gaps']) ? $settings['column_gaps'] : 'default',

                'source_settings' => array(
                    'feed_type' => isset($settings['source_settings']['feed_type']) ? $settings['source_settings']['feed_type'] : 'channel_feed',
                    'channel_id' => isset($settings['source_settings']['channel_id']) ? $settings['source_settings']['channel_id'] : '',
                    'playlist_id' => isset($settings['source_settings']['playlist_id']) ? $settings['source_settings']['playlist_id'] : '',
                    'user_name' => isset($settings['source_settings']['user_name']) ? $settings['source_settings']['user_name'] : '',
                    'feed_count' => isset($settings['source_settings']['feed_count']) ? intval($settings['source_settings']['feed_count']) : self::$totalFeed,
                    'search_term' => isset($settings['source_settings']['search_term']) ? $settings['source_settings']['search_term'] : '',
                    'video_id' => isset($settings['source_settings']['video_id']) ? $settings['source_settings']['video_id'] : '',
                    'event_type' => isset($settings['source_settings']['event_type']) ? $settings['source_settings']['event_type'] : 'completed',
                ),

                'video_settings' => array(
                    'display_play_icon' => isset($settings['video_settings']['display_play_icon']) ? $settings['video_settings']['display_play_icon'] : 'true',
                    'display_duration' => isset($settings['video_settings']['display_duration']) ? $settings['video_settings']['display_duration'] : 'true',
                    'display_title' => isset($settings['video_settings']['display_title']) ? $settings['video_settings']['display_title'] : 'true',
                    'trim_title_words' => isset($settings['video_settings']['trim_title_words']) ? intval($settings['video_settings']['trim_title_words']) : 8,
                    'display_date' => isset($settings['video_settings']['display_date']) ? $settings['video_settings']['display_date'] : 'true',
                    'display_description' => isset($settings['video_settings']['display_description']) ? $settings['video_settings']['display_description'] : 'true',
                    'display_views_counter' => isset($settings['video_settings']['display_views_counter']) ? $settings['video_settings']['display_views_counter'] : 'true',
                    'display_likes_counter' => isset($settings['video_settings']['display_likes_counter']) ? $settings['video_settings']['display_likes_counter'] : 'false',
                    'display_comments_counter' => isset($settings['video_settings']['display_comments_counter']) ? $settings['video_settings']['display_comments_counter'] : 'false',
                    'play_mode' => isset($settings['video_settings']['play_mode']) ? $settings['video_settings']['play_mode'] : 'inline',
                    'display_channel_name' => isset($settings['video_settings']['display_channel_name']) ? $settings['video_settings']['display_channel_name'] : 'false',
                ),

                'carousel_settings' => array(
                    'autoplay' => isset($settings['carousel_settings']['autoplay']) ? $settings['carousel_settings']['autoplay'] : 'true',
                    'autoplay_speed' => isset($settings['carousel_settings']['autoplay_speed']) ? intval($settings['carousel_settings']['autoplay_speed']) : 3000,
                    'rows' => isset($settings['carousel_settings']['rows']) ? intval($settings['carousel_settings']['rows']) : 1,
                    'slides_to_show' => isset($settings['carousel_settings']['slides_to_show']) ? intval($settings['carousel_settings']['slides_to_show']) : 3,
                    'slides_to_scroll' => isset($settings['carousel_settings']['slides_to_scroll']) ? intval($settings['carousel_settings']['slides_to_scroll']) : 3,
                    'navigation' => isset($settings['carousel_settings']['navigation']) ? $settings['carousel_settings']['navigation'] : 'dot',
                    'vertical' => isset($settings['carousel_settings']['vertical']) ? $settings['carousel_settings']['vertical'] : 'false',
                ),

                'header_settings' => array(

                    'display_header' => isset($settings['header_settings']['display_header']) ? $settings['header_settings']['display_header'] : 'true',
                    'display_name' => isset($settings['header_settings']['display_name']) ? $settings['header_settings']['display_name'] : 'true',
                    'display_description' => isset($settings['header_settings']['display_description']) ? $settings['header_settings']['display_description'] : 'false',
                    'display_videos_counter' => isset($settings['header_settings']['display_videos_counter']) ? $settings['header_settings']['display_videos_counter'] : 'true',
                    'display_subscriber_counter' => isset($settings['header_settings']['display_subscriber_counter']) ? $settings['header_settings']['display_subscriber_counter'] : 'true',
                    'display_views_counter' => isset($settings['header_settings']['display_views_counter']) ? $settings['header_settings']['display_views_counter'] : 'true',
                    'display_logo' => isset($settings['header_settings']['display_logo']) ? $settings['header_settings']['display_logo'] : 'true',
                    'display_banner' => isset($settings['header_settings']['display_banner']) ? $settings['header_settings']['display_banner'] : 'true',
                    'display_subscribe_button' => isset($settings['header_settings']['display_subscribe_button']) ? $settings['header_settings']['display_subscribe_button'] : 'true',

                ),

                'popup_settings' => array(

                    'display_title' => isset($settings['popup_settings']['display_title']) ? $settings['popup_settings']['display_title'] : 'true',
                    'display_channel_name' => isset($settings['popup_settings']['display_channel_name']) ? $settings['popup_settings']['display_channel_name'] : 'true',
                    'display_channel_logo' => isset($settings['popup_settings']['display_channel_logo']) ? $settings['popup_settings']['display_channel_logo'] : 'true',
                    'display_subscribers_counter' => isset($settings['popup_settings']['display_subscribers_counter']) ? $settings['popup_settings']['display_subscribers_counter'] : 'true',
                    'display_subscribe_button' => isset($settings['popup_settings']['display_subscribe_button']) ? $settings['popup_settings']['display_subscribe_button'] : 'true',
                    'display_views_counter' => isset($settings['popup_settings']['display_views_counter']) ? $settings['popup_settings']['display_views_counter'] : 'true',
                    'display_likes_counter' => isset($settings['popup_settings']['display_likes_counter']) ? $settings['popup_settings']['display_likes_counter'] : 'true',
                    'display_dislikes_counter' => isset($settings['popup_settings']['display_dislikes_counter']) ? $settings['popup_settings']['display_dislikes_counter'] : 'true',
                    //'display_likes_ratio'       => isset($settings['popup_settings']['display_likes_ratio']) ? $settings['popup_settings']['display_likes_ratio'] : 'true',
                    //'display_share'             => isset($settings['popup_settings']['display_share']) ? $settings['popup_settings']['display_share'] : 'true',
                    'display_date' => isset($settings['popup_settings']['display_date']) ? $settings['popup_settings']['display_date'] : 'true',
                    'display_description' => isset($settings['popup_settings']['display_description']) ? $settings['popup_settings']['display_description'] : 'true',
                    //'display_description_more'  => isset($settings['popup_settings']['display_description_more']) ? $settings['popup_settings']['display_description_more'] : 'true',
                    'display_comments' => isset($settings['popup_settings']['display_comments']) ? $settings['popup_settings']['display_comments'] : 'true',
                    'autoplay' => isset($settings['popup_settings']['autoplay']) ? $settings['popup_settings']['autoplay'] : 'true',
                    'video_loop' => isset($settings['popup_settings']['video_loop']) ? $settings['popup_settings']['video_loop'] : 'false',

                ),

                'filters' => array(
                    'total_posts' => isset($settings['filters']['total_posts']) ? intval($settings['filters']['total_posts']) : 50,
                    'post_order' => isset($settings['filters']['post_order']) ? $settings['filters']['post_order'] : 'ascending',
                    'includes_inputs' => isset($settings['filters']['includes_inputs']) ? $settings['filters']['includes_inputs'] : '',
                    'excludes_inputs' => isset($settings['filters']['excludes_inputs']) ? $settings['filters']['excludes_inputs'] : '',
                ),

                'pagination_settings' => array(

                    'pagination_type' => isset($settings['pagination_settings']['pagination_type']) ? $settings['pagination_settings']['pagination_type'] : 'none',
                    'paginate' => (isset($settings['pagination_settings']['paginate']) && !empty($settings['pagination_settings']['paginate'])) ? $settings['pagination_settings']['paginate'] : 6,

                ),

            ),

            'dynamic' => $response,

            'feed_info' => array(
                'feed_type' => isset($settings['source_settings']['feed_type']) ? $settings['source_settings']['feed_type'] : '',
                'event_type' => isset($settings['source_settings']['event_type']) ? $settings['source_settings']['event_type'] : '',
            ),

        );
    }

    /**
     * Update template is settings is changed in editor
     *
     * @return json
     * @since 1.2.5
     */
    public function saveUpdatedFeedEditorSettings()
    {
        $postId = intval($_REQUEST['post_id']);
        $args = json_decode(wp_unslash($_REQUEST['args']));
        $args = json_decode(json_encode($args), true);
        update_post_meta($postId, '_wpsr_youtube_template_config', $args);
        Helper::sendSuccess([
            'message' => __('Template Saved Successfully!!', 'wpsocialreviews'),
        ]);
    }

    /**
     * Show updated data if new data fetched in editor without update in db
     *
     * @return json
     * @throws /Exception
     * @since 1.2.5
     */
    public function updateFeedEditorSettings()
    {
        $postId = intval($_REQUEST['post_id']);
        $configs = json_decode(wp_unslash($_REQUEST['args']));
        $configs = json_decode(json_encode($configs), true);

        $formatted_feed_template_meta = self::formatYoutubeConfig($configs['feed_settings'], array());

        $feed_settings = isset($formatted_feed_template_meta['feed_settings']) ? $formatted_feed_template_meta['feed_settings'] : array();

        $feedType = isset($feed_settings['source_settings']['feed_type']) ? $feed_settings['source_settings']['feed_type'] : 'channel_feed';

        if ($feedType === 'single_video') {

            $feedCacheName = 'wpsr_youtube_videos_list_' . $postId;

            //if ( true === $this->cacheHandler->checkFeedCacheExists( $feedCacheName ) ) {

            // delete this cache
            wpSocialReviewsDB()->table('options')->where('option_name', 'LIKE', '_transient_wpsr_' . $this->platform . '_tc_' . $feedCacheName . '%')->delete();
            wpSocialReviewsDB()->table('options')->where('option_name', 'LIKE', '_transient_timeout_wpsr_' . $this->platform . '_tc_' . $feedCacheName . '%')->delete();
            wpSocialReviewsDB()->table('options')->where('option_name', 'LIKE', '_transient_wpsr_' . $this->platform . '_pc_' . $feedCacheName . '%')->delete();

            // }

        }

        $apiSettings = isset($feed_settings['source_settings']) ? $feed_settings['source_settings'] : array();
        $response = $this->apiConnection($apiSettings, $postId);

        $filterSettings = isset($feed_settings['filters']) ? $feed_settings['filters'] : array();
        $filterResponse = array();

        if (isset($response['error_message'])) {
            $filterResponse = $response;
        } else {
            $filterResponse = (new FeedFilters)->filterFeedResponse($this->platform, $filterSettings, $response);
        }

        $formatted_feed_template_meta['dynamic'] = $filterResponse;
        $configs = $formatted_feed_template_meta;

        Helper::sendSuccess([
            'message' => __('Youtube Settings Updated', 'wpsocialreviews'),
            'settings' => $configs
        ]);
    }

    /**
     * Save YouTube config on wp options table
     *
     * @return json
     * @since 1.2.5
     */
    public function setYoutubeAccountData()
    {
        // verify access token and API key
        try {
            $credentialsType = sanitize_text_field($_REQUEST['credentialsType']);
            $apiKey = sanitize_text_field($_REQUEST['apiKey']);
            $accessCode = sanitize_text_field($_REQUEST['accessCode']);
            $accessToken = sanitize_text_field($_REQUEST['accessToken']);
            $url = $this->makeConfigAPIRequestUrl($credentialsType, $accessCode, $apiKey, $accessToken);
            $data = $this->getAPIData($url['fetchUrl'], $url['args']);

            if (isset($data['error']) || !empty($data['error'])) {
                $code = isset($data['error']['code']) ? $data['error']['code'] : 401;
                wp_send_json_error([
                    'settings' => $data,
                    'message' => isset($data['error']['errors'][0]) ? $data['error']['errors'][0]['message'] : __('Sorry, Something went wrong!', 'wpsocialreviews'),
                ], $code);
            }

            $configs = array();
            if ((!empty($data) && isset($data['items'][0])) && ($credentialsType === 'oauth2.0' || $credentialsType === 'manually_connect')) {
                $configs['channel_id'] = $data['items'][0]['id'];
                $configs['user_name'] = $data['items'][0]['snippet']['title'];
            }
            $configs['access_token'] = isset($url['accessToken']) ? $url['accessToken'] : '';
            $configs['refresh_token'] = isset($url['refresh_token']) ? $url['refresh_token'] : '';
            $configs['expires_in'] = isset($url['expires_in']) ? $url['expires_in'] : '';
            $configs['api_key'] = $apiKey;
            $configs['access_code'] = $accessCode;
            $configs['credentials_type'] = $credentialsType;
            $configs['created_at'] = time();
            update_option('wpsr_youtube_verification_configs', $configs, 'no');
            $settings = get_option('wpsr_youtube_verification_configs');
            // add global youtube settings when user verified
            $args = array(
                'global_settings' => array(
                    'feed_cache_time' => '1209600',
                    'feed_cache_clear' => 'clear_cache_button',
                    'auto_syncing' => 'true',
                    'cache_recurrence' => '2weeks'
                )
            );
            update_option('wpsr_youtube_global_settings', $args);
            $this->doCronEvent();
            wp_send_json_success([
                'settings' => $settings,
                'message' => __('You are Successfully Verified', 'wpsocialreviews')
            ]);
        } catch (\Exception $exception) {
            wp_send_json_error([
                'message' => $exception->getMessage()
            ], 423);
        }
    }

    /**
     * Make api request url base on credentials type
     *
     * @param string $credentialsType
     * @param string $accessCode
     * @param string $apiKey
     * @param string $accessToken
     * @return array
     * @throws /Exception
     * @since 1.2.5
     */
    private function makeConfigAPIRequestUrl($credentialsType = '', $accessCode = '', $apiKey = '', $accessToken)
    {
        $url = [];
        if (($credentialsType === 'oauth2.0' && empty($accessCode)) || ($credentialsType === 'api_key' && empty($apiKey)) || ($credentialsType === 'manually_connect' && empty($accessToken))) {
            wp_send_json_error([
                'message' => __('Field should not be empty!', 'wpsocialreviews')
            ], 423);
        }
        if (($credentialsType === 'oauth2.0' || $credentialsType === 'manually_connect')) {
            if ($accessCode && $credentialsType === 'oauth2.0') {
                $result = $this->oauth->generateAccessKey($accessCode);
                if (is_wp_error($result)) {
                    throw new \Exception($result->get_error_message());
                }
                $tokens['credentials_type'] = 'oauth2.0';
                $accessToken = $result['access_token'];
                $refreshToken = $result['refresh_token'];
                $expireTime = $result['expires_in'];
                $url['expires_in'] = $expireTime;
                $url['refresh_token'] = $refreshToken;
            }
            $args = array(
                'headers' => array(
                    'Authorization' => 'Bearer ' . $accessToken,
                ),
            );
            $url['args'] = $args;
            $url['accessToken'] = $accessToken;
            $part = 'mine=true&access_token=' . $accessToken;
            $fetchUrl = 'https://www.googleapis.com/youtube/v3/channels?part=id,snippet&contentDetails&' . $part;
            $url['fetchUrl'] = $fetchUrl;
        }
        if ($credentialsType === 'api_key' && $apiKey) {
            $part = 'key=' . $apiKey;
            $fetchUrl = 'https://www.googleapis.com/youtube/v3/search?part=snippet&q=YouTube+Data+API&type=video&' . $part;
            $url['fetchUrl'] = $fetchUrl;
            $url['args'] = [];
        }
        return $url;
    }

    /**
     * This function will set the API url and fetch the YouTube API Data
     *
     * @param string $fetchUrl
     * @param array $args
     * @return json
     * @since 1.2.5
     */
    public function getAPIData($fetchUrl = '', $args = array())
    {
        $response = wp_remote_get($fetchUrl, $args);
        $response = json_decode(wp_remote_retrieve_body($response), true);
        return $response;
    }

    /**
     * get verified youtube configs data from wp options table
     *
     * @return json
     * @since 1.2.5
     */
    public function getVerificationConfigs()
    {
        $configs = get_option('wpsr_youtube_verification_configs');
        Helper::sendSuccess([
            'message' => __('Youtube Successfully Connected!', 'wpsocialreviews'),
            'settings' => $configs
        ]);
    }

    /**
     * delete youtube configs data from wp options table
     *
     * @return json
     * @since 1.2.5
     */
    public function disconnectVerificationConfigs()
    {
        delete_option('wpsr_youtube_verification_configs');
        delete_option('wpsr_youtube_global_settings');
        Helper::sendSuccess([
            'message' => __('YouTube Successfully Disconnected!', 'wpsocialreviews'),
        ]);
    }

    /**
     * Get Cache Settings
     *
     * @return json
     * @since 1.2.5
     */
    public function getGlobalSettings()
    {
        $settings = get_option('wpsr_youtube_global_settings', false);
        Helper::sendSuccess([
            'message' => __('Settings Saved Successfully', 'wpsocialreviews'),
            'settings' => $settings
        ]);
    }

    /**
     * Save Cache Settings
     *
     * @return json
     * @since 1.2.5
     */
    public function saveGlobalSettings()
    {
        $args = json_decode(wp_unslash($_REQUEST['args']));
        $args = json_decode(json_encode($args), true);
        update_option('wpsr_youtube_global_settings', $args, 'no');
        $settings = get_option('wpsr_youtube_global_settings', false);
        $this->doCronEvent();
        Helper::sendSuccess([
            'message' => __('Settings Saved Successfully', 'wpsocialreviews'),
            'settings' => $settings
        ]);
    }

    /**
     * Set cron event
     *
     * @since 1.2.5
     */
    public function doCronEvent()
    {
        $settings = get_option('wpsr_youtube_global_settings', false);
        if ($settings) {
            //check sync is on or off, if off clear cron event
            $sync = isset($settings['global_settings']['auto_syncing']) ? $settings['global_settings']['auto_syncing'] : 'true';
            $recurrence = isset($settings['global_settings']['cache_recurrence']) ? $settings['global_settings']['cache_recurrence'] : '2weeks';
            if ($sync === 'false') {
                // clear cron event
                wp_clear_scheduled_hook('wpsr_daily_feed_event_' . $this->platform);
            } else if ($sync === 'true') {
                //clear the existing cron event
                wp_clear_scheduled_hook('wpsr_daily_feed_event_' . $this->platform);
                wp_schedule_event(time(), $recurrence, 'wpsr_daily_feed_event_' . $this->platform);
            }
        }
    }

}
