<?php

namespace WPSocialReviews\Classes;

use WPSocialReviews\Classes\Builder\Render;
use WPSocialReviews\Classes\Models\ReviewsModel;
use WPSocialReviews\Classes\Feeds\Twitter\TwitterFeed;
use WPSocialReviews\Classes\Feeds\Youtube\YoutubeFeed;
use WPSocialReviews\Classes\Feeds\Common\FeedFilters;

//instagram configs
use WPSocialReviews\Classes\Feeds\Instagram\InstagramFeed;

if (!defined('ABSPATH')) {
    exit;
}

class FrontEndAjaxHandler
{
    /**
     *
     * Retrieve reviews load more data
     *
     * @since 1.2.4
     *
     **/
    public function loadMoreReviewsAjaxHandler()
    {
        check_ajax_referer('wpsr-ajax-nonce', 'security');
        $templateMapping = apply_filters('wpsocialreviews/add_reviews_template',
            [
                'grid1' => 'reviews-templates/template1',
                'grid2' => 'reviews-templates/template2',
                'grid3' => 'reviews-templates/template3',
                'grid4' => 'reviews-templates/template4',
                'grid5' => 'reviews-templates/template5',
            ]
        );
        $templateId = intval($_POST['templateId']);
        $page = intval($_POST['page']) + 1;
        $template_meta = get_post_meta($templateId, '_wpsr_template_config', true);
        $reviews = array();
        $platforms = isset($template_meta['platform']) ? $template_meta['platform'] : array();
        $activePlatforms = apply_filters('wpsocialreviews/available_valid_platforms', []);
        $validTemplatePlatforms = array_intersect($platforms, array_keys($activePlatforms));

        $isCustomReviewsExists = ReviewsModel::isCustomFound();
        if ($isCustomReviewsExists && in_array('custom', $platforms)) {
            $validTemplatePlatforms['custom'] = __('Custom', 'wpsocialreviews');
        }

        $data = ReviewsModel::getPaginatedFilteredReviewsByPlatforms($validTemplatePlatforms, $template_meta, $page);
        $reviews = $data['reviews'];

        $template = $template_meta['template'];
        if (!isset($templateMapping[$template])) {
            return __('No Templates found!! Please save and try again', 'wpsocialreviews');
        }
        $file = $templateMapping[$template];
        $html = '';
        $html .= Render::loadView($file, array(
            'reviews' => $reviews,
            'template_meta' => $template_meta
        ));
        echo $html;
        die();
    }

    /**
     *
     * Retrieve youtube load more data
     *
     * @since 1.2.5
     *
     **/
    public function loadMoreYouTubeAjaxHandler()
    {
        check_ajax_referer('wpsr-ajax-nonce', 'security');
        $templateId = intval($_POST['templateId']);
        $page = intval($_POST['page']);

        $youtube_feed = new YoutubeFeed();
        $feed_template_meta = get_post_meta($templateId, '_wpsr_youtube_template_config', true);

        $configs = isset($feed_template_meta['feed_settings']) ? $feed_template_meta['feed_settings'] : array();
        $formatted_feed_template_meta = $youtube_feed->formatYoutubeConfig($configs, array());

        $feed_settings = isset($formatted_feed_template_meta['feed_settings']) ? $formatted_feed_template_meta['feed_settings'] : array();

        $apiSettings = isset($feed_settings['source_settings']) ? $feed_settings['source_settings'] : array();
        $filterSettings = isset($feed_settings['filters']) ? $feed_settings['filters'] : array();

        $response = $youtube_feed->apiConnection($apiSettings, $templateId);

        $filterResponse = array();
        if (isset($response['error_message'])) {
            $filterResponse = $response;
        } else {
            $filterResponse = (new FeedFilters)->filterFeedResponse('youtube', $filterSettings, $response);
        }

        $feeds = isset($filterResponse['items']) ? $filterResponse['items'] : array();

        $feed_info = isset($formatted_feed_template_meta['feed_info']) ? $formatted_feed_template_meta['feed_info'] : array();

        $templateMapping = apply_filters('wpsocialreviews/add_youtube_template',
            [
                'template1' => 'feed-templates/youtube/template1',
                'template2' => 'feed-templates/youtube/template2',
                'template3' => 'feed-templates/youtube/template3',
            ]
        );
        $template = isset($feed_settings['template']) ? $feed_settings['template'] : '';
        // render template 1, template 2, template 3 from template 1
        $template = $template === 'template2' || $template === 'template3' ? 'template1' : $template;
        if (!isset($templateMapping[$template])) {
            return __('No Templates found!! Please save and try again', 'wpsocialreviews');
        }
        $file = $templateMapping[$template];
        $paginate = intval($feed_settings['pagination_settings']['paginate']);
        $sinceId = ($page * $paginate);
        $maxId = ($sinceId + $paginate) - 1;
        $html = '';
        $html .= Render::loadView($file, array(
            'templateId' => $templateId,
            'feeds' => $feeds,
            'template_meta' => $feed_settings,
            'paginate' => $paginate,
            'sinceId' => $sinceId,
            'maxId' => $maxId,
            'feed_info' => $feed_info
        ));
        echo $html;
        die();
    }

    /**
     *
     * Retrieve twitter load more data
     *
     * @since 1.2.4
     *
     **/
    public function loadMoreTwitterAjaxHandler()
    {
        check_ajax_referer('wpsr-ajax-nonce', 'security');
        $templateId = intval($_POST['templateId']);
        $page = intval($_POST['page']);
        $feed_template_meta = get_post_meta($templateId, '_wpsr_twitter_template_config', true);

        if ($feed_template_meta) {

            $twitter_feed = new TwitterFeed();

            $configs = isset($feed_template_meta['feed_settings']) ? $feed_template_meta['feed_settings'] : array();
            $formatted_feed_template_meta = $twitter_feed->formatTwitterConfig($configs, array());

            //feed settings to simplify repetitive use
            $feed_settings = isset($formatted_feed_template_meta['feed_settings']) ? $formatted_feed_template_meta['feed_settings'] : array();

            $paginate = intval($feed_settings['pagination_settings']['paginate']);
            $sinceId = ($page * $paginate);
            $maxId = ($sinceId + $paginate) - 1;

            //get dynamic response
            $apiSettings = isset($feed_settings['additional_settings']) ? $feed_settings['additional_settings'] : array();

            $response = $twitter_feed->apiConnectionResponse($apiSettings['feed_type'], $apiSettings['feed_count'], $feed_template_meta);

            //filter the feeds
            $filterSettings = isset($feed_settings['filters']) ? $feed_settings['filters'] : array();
            $filterResponse = array();
            if (isset($response['error_message'])) {
                $filterResponse = $response;
            } else {
                $filterResponse = (new FeedFilters)->filterFeedResponse('twitter', $filterSettings, $response);
            }

            $feeds = isset($filterResponse['feed']) ? $filterResponse['feed'] : $filterResponse;

            $html = '';
            if (isset($feeds['error_message'])) {
                $html .= $feeds['error_message'];
            } else {
                $template = isset($feed_settings['layout_type']) && $feed_settings['layout_type'] !== 'standard' ? 'template2' : 'template1';
                $html .= Render::loadView('feed-templates/twitter/' . $template . '', array(
                    'feeds' => $feeds,
                    'templateId' => $templateId,
                    'template_meta' => $feed_settings,
                    'paginate' => $paginate,
                    'sinceId' => $sinceId,
                    'maxId' => $maxId,
                ));
            }
            echo $html;
            die();
        }

    }

}
