<?php

namespace WPSocialReviews\Classes\Models;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Base Model Class
 * @since 1.0.0
 */
class ReviewsModel
{

    //all filtered reviews
    public static function getFilteredReviewsByPlatforms($platforms, $filters)
    {
        $reviews = array();

        $includeIds = isset($filters['selectedIncList']) ? $filters['selectedIncList'] : array();
        $excludeIds = isset($filters['selectedExcList']) ? $filters['selectedExcList'] : array();

        $totalReviews  = isset($filters['totalReviewsVal']) ? intval($filters['totalReviewsVal']) : -1;
        $starFilterVal = isset($filters['starFilterVal']) ? intval($filters['starFilterVal']) : 6;
        $filterByTitle = isset($filters['filterByTitle']) ? $filters['filterByTitle'] : 'all';
        $order  = isset($filters['dateFilterVal']) ? $filters['dateFilterVal'] : 'none';

        $reviews = wpSocialReviewsDB()
            ->table('wpsr_reviews')
            ->whereIn('platform_name', $platforms);

        if( $order !== 'none' ) {
            $reviews = $reviews->orderBy('review_time', $order);
        }

        //filter by star rating
        if ($starFilterVal < 6) {
            $reviews = $reviews->where('rating', '>=', $filters['starFilterVal']);
        }

        //filter by included or excluded
        if ($filterByTitle === 'include' && count($includeIds)) {
            $reviews = $reviews->whereIn('id', $includeIds);
        }

        if (count($excludeIds) && $filterByTitle === 'exclude') {
            $reviews = $reviews->whereNotIn('id', $excludeIds);
        }

        //filtered by total reviews
        if ($totalReviews > -1) {
            $reviews = $reviews->limit($totalReviews);
        }

        $reviews = $reviews->get();
        return $reviews;
    }

    /**
     * insert new reviews
     * @param $reviews
     * @return boolean
     */
    public static function create($reviews)
    {
        return wpSocialReviewsDB()->table('wpsr_reviews')
            ->insert($reviews);
    }

    public static function isCustomFound()
    {
        return wpSocialReviewsDB()->table('wpsr_reviews')
            ->select('*')
            ->where('platform_name', 'custom')
            ->count();
    }

    public function getAllReviews()
    {
        return wpSocialReviewsDB()
            ->table('wpsr_reviews')
            ->select('*')
            ->count();
    }

    public static function getPaginatedFilteredReviewsByPlatforms($platforms, $filters = array(), $page = 1)
    {
        $reviews = array();

        $paginate = isset($filters['paginate']) ? intval($filters['paginate']) : 6;
        $offset = ($paginate * $page) - $paginate;

        $includeIds = isset($filters['selectedIncList']) ? $filters['selectedIncList'] : array();
        $excludeIds = isset($filters['selectedExcList']) ? $filters['selectedExcList'] : array();

        $totalFilterReviews = isset($filters['totalReviewsVal']) ? intval($filters['totalReviewsVal']) : -1;

        $starFilterVal = isset($filters['starFilterVal']) ? intval($filters['starFilterVal']) : 6;
        $filterByTitle = isset($filters['filterByTitle']) ? $filters['filterByTitle'] : 'all';

        $paginationType = isset($filters['pagination_type']) ? $filters['pagination_type'] : '';
        $templateType = isset($filters['templateType']) ? $filters['templateType'] : 'grid';

        $order  = isset($filters['dateFilterVal']) ? $filters['dateFilterVal'] : 'none';

        $reviews = wpSocialReviewsDB()
            ->table('wpsr_reviews')
            ->whereIn('platform_name', $platforms);

        if( $order !== 'none' ) {
            $reviews = $reviews->orderBy('review_time', $order);
        }

        //filter by star rating
        if ($starFilterVal < 6) {
            $reviews = $reviews->where('rating', '>=', $filters['starFilterVal']);
        }

        //filter by included or excluded
        if ($filterByTitle === 'include' && count($includeIds)) {
            $reviews = $reviews->whereIn('id', $includeIds);
        }

        if (count($excludeIds) && $filterByTitle === 'exclude') {
            $reviews = $reviews->whereNotIn('id', $excludeIds);
        }

        //filtered by total reviews
        if ($totalFilterReviews > -1) {
            $reviews = $reviews->limit($totalFilterReviews);
        }

        $totalReviews = $reviews->count();

        // activate pagination
        if ($paginationType === 'load_more' && $templateType !== 'slider') {

            if( $totalFilterReviews > -1 ) { 

                if( $totalFilterReviews < $paginate ) {
                    $paginate = $totalFilterReviews;
                }else {

                    $reviewsNow    = $page * $paginate ;

                    if($reviewsNow>$totalFilterReviews) {
                        $extraReviews = ($reviewsNow-$totalFilterReviews); 
                        $paginate = $paginate-$extraReviews;
                    }

                }

            }

            $reviews = $reviews->orderby('id', 'asc')
                ->offset($offset)
                ->limit($paginate);
        }

        $reviews = $reviews->get();

        $data = array(
            'total_reviews' => $totalReviews,
            'reviews' => $reviews
        );
        return $data;
    }

    /**
     * Check is platform reviews exists or not
     * @param $platform
     * @param $source_id
     * @return object
     */
    public static function existData($platform, $source_id)
    {
        return wpSocialReviewsDB()->table('wpsr_reviews')
            ->where('platform_name', $platform)
            ->first();
    }

    /**
     * Check is platform reviews exists or not
     * @param $platform
     * @param $source_id
     * @return object
     */
    public static function getReviewByPlatformSource($platform, $source_id)
    {
        return wpSocialReviewsDB()->table('wpsr_reviews')
            ->where('platform_name', $platform)
            ->where('source_id', $source_id)
            ->first();
    }

    public static function getAllQueryReviews($args = array())
    {
        $templateQuery = wpSocialReviewsDB()->table('wpsr_reviews')
            ->orderBy('id', 'DESC')
            ->offset($args['offset'])
            ->limit($args['posts_per_page']);

        if (!empty($args['s'])) {
            $templateQuery->where(function ($q) use ($args) {
                $q->where('platform_name', 'LIKE', "%{$args['s']}%");
                $q->orWhere('id', 'LIKE', "%{$args['s']}%");
                $q->orWhere('review_title', 'LIKE', "%{$args['s']}%");
                $q->orWhere('reviewer_name', 'LIKE', "%{$args['s']}%");
                $q->orWhere('reviewer_text', 'LIKE', "%{$args['s']}%");
                $q->orWhere('review_time', 'LIKE', "%{$args['s']}%");
                $q->orWhere('rating', 'LIKE', "%{$args['s']}%");
            });
        }

        $total = $templateQuery->count();
        $templates = $templateQuery->get();

        //$templates = apply_filters('wpsocialreviews/get_all_templates', $templates);

        $lastPage = ceil($total / $args['posts_per_page']);

        return array(
            'templates' => $templates,
            'total' => $total,
            'last_page' => $lastPage
        );
    }

    public static function getReviewById($reviewId)
    {
        $review = wpSocialReviewsDB()->table('wpsr_reviews')
            ->where('id', $reviewId)
            ->first();

        return $review;
    }

    public static function updateReview($reviewId, $reviewData)
    {
        $review = wpSocialReviewsDB()->table('wpsr_reviews')
            ->where('id', $reviewId)
            ->update([

                'platform_name' => $reviewData['review_platform'],
                'reviewer_name' => $reviewData['reviewer_name'],
                'reviewer_text' => $reviewData['reviewer_text'],
                'review_time' => $reviewData['review_time'],
                'rating' => $reviewData['reviewer_rating'],
                'reviewer_img' => $reviewData['reviewer_image'],

            ]);

        return $reviewData;
    }

    public static function deleteReview($reviewData)
    {
        $review = wpSocialReviewsDB()->table('wpsr_reviews')
            ->where('id', $reviewData['id'])
            ->delete();

        return $review;
    }

    /**
     * Return all reviews
     *
     * @param $platform
     * @param $source_id
     *
     * @return array
     * @throws \WpFluent\Exception
     */
    public static function getReviews($platform, $source_id)
    {
        $reviews = wpSocialReviewsDB()->table('wpsr_reviews')
            ->where('platform_name', $platform)
            ->get();
        return $reviews;
    }

    /**
     * Update old reviews while found new reviews
     *
     * @param $id
     * @param $reviews
     *
     * @return boolean
     * @throws \WpFluent\Exception
     */
    public static function updateReviews($id, $reviews)
    {
        return wpSocialReviewsDB()->table('wpsr_reviews')
            ->where('id', $id)
            ->update($reviews);
    }

    /**
     * Return Platform Name For single Platform
     * @param $platform
     * @param
     * @return object|string
     */
    public static function getPlatformName($platform)
    {
        $platform = wpSocialReviewsDB()->table('wpsr_reviews')
            ->select('platform_name')
            ->where('platform_name', $platform)
            ->first();
        return $platform;
    }

    /**
     * Return multiple reviews for multiple platforms
     *
     * @param $platforms
     *
     * @return array
     * @throws \WpFluent\Exception
     */

    public static function getMultipleReviews($platforms)
    {
        $reviewsQuery = wpSocialReviewsDB()->table('wpsr_reviews');
        foreach ($platforms as $key => $where) {
            if ($key == 0) {
                $reviewsQuery->where('platform_name', $where);
            } else {
                $reviewsQuery->Orwhere('platform_name', $where);
            }
        }
        $reviews = $reviewsQuery->get();
        return $reviews;
    }

    public static function duplicateReview($review)
    {
        $duplicate = [
            'platform_name' => $review['platform_name'],
            'source_id' => 'custom',
            'reviewer_name' => $review['reviewer_name'],
            'review_title' => '(Duplicate)' . $review['review_title'] . ' (#' . $review['id'] . ')',
            'reviewer_url' => $review['reviewer_url'],
            'reviewer_img' => $review['reviewer_img'],
            'reviewer_text' => $review['reviewer_text'],
            'rating' => $review['rating'],
            'review_time' => $review['review_time'],
            'recommendation_type' => 'positive',
            'updated_at' => date('Y-m-d H:i:s'),
        ];

        $insertReview = wpSocialReviewsDB()->table('wpsr_reviews')->insert($duplicate);

        return $insertReview;
    }

    public static function formattedTemplateMeta($settings)
    {

        $meta = array(

            //source
            'platform' => isset($settings['platform']) ? $settings['platform'] : array(),

            //template
            'platformType' => isset($settings['platformType']) ? $settings['platformType'] : 'single',
            'template' => isset($settings['template']) ? $settings['template'] : 'grid1',
            'templateType' => isset($settings['templateType']) ? $settings['templateType'] : 'grid',
            'column' => isset($settings['column']) ? intval($settings['column']) : 4,

            //settings
            'reviewer_name' => isset($settings['reviewer_name']) ? $settings['reviewer_name'] : 'true',
            'rating_style' => isset($settings['rating_style']) ? $settings['rating_style'] : 'default',
            'reviewer_image' => isset($settings['reviewer_image']) ? $settings['reviewer_image'] : 'true',
            'timestamp' => isset($settings['timestamp']) ? $settings['timestamp'] : 'true',
            'reviewerrating' => isset($settings['reviewerrating']) ? $settings['reviewerrating'] : 'true',

            'equal_height' => isset($settings['equal_height']) ? $settings['equal_height'] : 'false',
            'equalHeightLen' => isset($settings['equalHeightLen']) ? intval($settings['equalHeightLen']) : 250,
            'content_length' => isset($settings['content_length']) ? intval($settings['content_length']) : 100,
            'showContentLength' => isset($settings['showContentLength']) ? $settings['showContentLength'] : 'true',

            'isReviewerText' => isset($settings['isReviewerText']) ? $settings['isReviewerText'] : 'true',
            'isPlatformIcon' => isset($settings['isPlatformIcon']) ? $settings['isPlatformIcon'] : 'true',
            'current_template_type' => isset($settings['current_template_type']) ? $settings['current_template_type'] : 'grid',

            //carousel
            'autoplay' => isset($settings['autoplay']) ? $settings['autoplay'] : 'false',
            'autoplay_speed' => isset($settings['autoplay_speed']) ? intval($settings['autoplay_speed']) : 2000,
            'slides_to_show' => isset($settings['slides_to_show']) ? intval($settings['slides_to_show']) : 3,
            'slides_to_scroll' => isset($settings['slides_to_scroll']) ? intval($settings['slides_to_scroll']) : 3,
            'slider_navigation_type' => isset($settings['slider_navigation_type']) ? $settings['slider_navigation_type'] : 'dot',

            //filters
            'totalReviewsVal' => isset($settings['totalReviewsVal']) ? intval($settings['totalReviewsVal']) : 10,
            'starFilterVal' => isset($settings['starFilterVal']) ? intval($settings['starFilterVal']) : 6,
            'filterByTitle' => isset($settings['filterByTitle']) ? $settings['filterByTitle'] : 'all',
            'selectedIncList' => isset($settings['selectedIncList']) ? $settings['selectedIncList'] : [],
            'selectedExcList' => isset($settings['selectedExcList']) ? $settings['selectedExcList'] : [],
            'dateFilterVal'   => isset($settings['dateFilterVal']) ? $settings['dateFilterVal'] : 'none',

            //header
            'show_header' => isset($settings['show_header']) ? $settings['show_header'] : 'false',
            'display_header_business_name' => isset($settings['display_header_business_name']) ? (bool)$settings['display_header_business_name'] : true,
            'display_header_rating' => isset($settings['display_header_rating']) ? (bool)$settings['display_header_rating'] : true,
            'display_header_reviews' => isset($settings['display_header_reviews']) ? (bool)$settings['display_header_reviews'] : true,
            'display_header_write_review' => isset($settings['display_header_write_review']) ? (bool)$settings['display_header_write_review'] : true,

            //pagination
            'pagination_type' => isset($settings['pagination_type']) ? $settings['pagination_type'] : 'none',
            'paginate' => (isset($settings['paginate']) && !empty($settings['paginate'])) ? intval($settings['paginate']) : 6,

            //styles
            'template_width' => isset($settings['template_width']) ? $settings['template_width'] : 'initial',
            'template_height' => isset($settings['template_height']) ? $settings['template_height'] : 'auto',
        );

        return $meta;
    }
}
