<?php

namespace WPSocialReviews\Classes\Models;

use WPSocialReviews\Classes\Helper;
use WPSocialReviews\Classes\PostType;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Templates Class
 * @since 1.0.0
 */
class Templates
{

    /**
     *
     * get all templates from posts table
     * @param $args
     * @return array
     * @throws /Exception
     * @since 1.0.0
     *
     **/
    public static function getTemplates($args = array(), $with = array())
    {
        $whereArgs = array(
            'post_type' => PostType::$CPTName,
            'post_status' => 'publish',
        );

        $whereArgs = apply_filters(
            'social_reviews/all_templates_where_args',
            $whereArgs
        );

        $templateQuery = wpSocialReviewsDB()->table('posts')
            ->orderBy('ID', 'DESC')
            ->offset($args['offset'])
            ->limit($args['posts_per_page']);

        foreach ($whereArgs as $key => $where) {
            $templateQuery->where($key, $where);
        }

        if (!empty($args['s'])) {
            $templateQuery->where(function ($q) use ($args) {
                $q->where('post_title', 'LIKE', "%{$args['s']}%");
                $q->orWhere('ID', 'LIKE', "%{$args['s']}%");
            });
        }

        $total = $templateQuery->count();
        $templates = $templateQuery->get();

        foreach ($templates as $template) {
            if (in_array('platform_name', $with)) {
                $template->platform_name = Helper::getPlatformNameByPostId($template->ID);
            }
        }

        $templates = apply_filters('wpsocialreviews/get_all_templates', $templates);

        $lastPage = ceil($total / $args['posts_per_page']);

        return array(
            'templates' => $templates,
            'total' => $total,
            'last_page' => $lastPage
        );
    }

    /**
     *
     * Delete template and meta from posts, post meta table
     * @param $templateId
     * @since 1.0.0
     *
     **/
    public static function deleteTemplate($templateId)
    {
        wp_delete_post($templateId, true);
        delete_post_meta($templateId, '_wpsr_template_config', true);
        delete_post_meta($templateId, '_wpsr_twitter_template_config', true);
        delete_post_meta($templateId, '_wpsr_youtube_template_config', true);
        delete_post_meta($templateId, '_wpsr_instagram_template_config', true);
    }

    /**
     *
     * save and update template title on post table
     * @param $templateId
     * @param $templateTitle
     * @return boolean
     * @since 1.0.0
     *
     **/
    public static function saveTitle($templateId, $templateTitle)
    {
        $updateArgs = array(
            'ID' => $templateId,
            'post_title' => $templateTitle,
        );

        $args = apply_filters('wpsocialreviews/template_title', $updateArgs);

        return wp_update_post($args);
    }


    /**
     *
     * get single template by templateId from posts table
     * @param $templateId
     * @return array
     * @since 1.0.0
     *
     **/
    public static function getTemplate($templateId)
    {
        $template = get_post($templateId, 'ARRAY_A');

        if (!$template || $template['post_type'] != PostType::$CPTName) {
            return false;
        }

        $template_config = get_post_meta($templateId, '_wpsr_template_config', true);
        if (!$template_config) {
            $template_config = get_post_meta($templateId, '_wpsr_twitter_template_config', true);
        }
        if (!$template_config) {
            $template_config = get_post_meta($templateId, '_wpsr_youtube_template_config', true);
        }
        if (!$template_config) {
            $template_config = get_post_meta($templateId, '_wpsr_instagram_template_config', true);
        }

        $template['template_meta'] = $template_config;

        return $template;
    }

    /**
     *
     * Create single template on posts table and save template meta by post id on postmeta table
     * @param $template
     * @return array
     * @since 1.0.0
     *
     **/
    public static function createNewTemplateData($template)
    {
        // Create the template post
        $post = wp_insert_post(array(
            'post_title' => $template['post_title'],
            'post_content' => $template['post_content'],
            'post_type' => PostType::$CPTName,
            'post_status' => 'publish',
            'post_author' => get_current_user_id()
        ));

        $template_config = get_post_meta($template['ID'], '_wpsr_template_config', true);

        if ($template_config) {
            update_post_meta($post, '_wpsr_template_config', $template_config);
        }

        $template_config = get_post_meta($template['ID'], '_wpsr_twitter_template_config', true);
        if (!$template_config) {
            $template_config = get_post_meta($template['ID'], '_wpsr_youtube_template_config', true);
        }
        if (!$template_config) {
            $template_config = get_post_meta($template['ID'], '_wpsr_instagram_template_config', true);
        }

        $platform = isset($template_config['feed_settings']['platform']) ? $template_config['feed_settings']['platform'] : '';

        if ($platform) {
            update_post_meta($post, '_wpsr_' . $platform . '_template_config', $template_config);
        }

        return static::getTemplate($template['ID']);
    }

}
