<?php

namespace WPSocialReviews\Classes\Reviews;

use WPSocialReviews\Classes\ArrayHelper;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Handle Airbnb Reviews
 * @since 1.0.0
 */
class Airbnb extends BaseReview
{
    private $remoteBaseUrl = 'https://www.airbnb.com/api/v2/';
    private $curUrl = 'https://www.airbnb.com';

    private $optionKey = 'social_review_airbnb_settings';

    public function __construct()
    {
        parent::__construct('airbnb');
        ini_set('memory_limit', '600M');
    }

    public function handleCredentialSave()
    {
        $downloadUrl = sanitize_text_field($_REQUEST['url_value']);
        try {
            $businessInfo = $this->verifyCredential($downloadUrl);
            update_option('social_review_airbnb_business_info', $businessInfo, 'no');
            wp_send_json_success([
                'message' => __('Airbnb Reviews successfully saved', 'wpsocialreviews'),
                'business_info' => $businessInfo
            ], 200);
        } catch (\Exception $exception) {
            wp_send_json_error([
                'message' => $exception->getMessage()
            ], 423);
        }
    }

    //cron job function
    public function wpsrDoThisCronEvent()
    {
        $settings = get_option($this->optionKey);
        $businessInfo = $this->verifyCredential($settings['url_value']);
        update_option('social_review_airbnb_business_info', $businessInfo, 'no');
    }

    public function pushValidPlatform($platforms)
    {
        $settings = $this->getApiSettings();
        if ($settings['api_key'] && $settings['place_id']) {
            $platforms['airbnb'] = __('Airbnb', 'wpsocialreviews');
        }
        return $platforms;
    }

    //for using curl instead of fopen
    public function wpsr_file_get_contents_curl_browser($url, $cookieval)
    {
        $agent = 'Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:60.0) Gecko/20100101 Firefox/60.0';
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_VERBOSE, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_USERAGENT, $agent);
        curl_setopt($ch, CURLOPT_ENCODING, 'gzip, deflate');
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            //'authority: nextdoor.com',//default
            'authority: gutendev.com',
            'pragma: no-cache',
            'cache-control: no-cache',
            'upgrade-insecure-requests: 1',
            'user-agent: Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36',
            'accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3', 'accept-encoding: gzip, deflate, br',
            'accept-language: en-US,en;q=0.9',
            'cookie: ' . $cookieval . ''
        ));
        curl_setopt($ch, CURLOPT_URL, $url);
        $result = curl_exec($ch);
        curl_close($ch);
        return $result;
    }

    public function verifyCredential($downloadUrl)
    {
        if (empty($downloadUrl)) {
            wp_send_json_error([
                'message' => __('Url field should not be empty!', 'wpsocialreviews'),
            ], 423);
        }

        if (filter_var($downloadUrl, FILTER_VALIDATE_URL)) {
            $this->curUrl = $downloadUrl;
            //start: find api key and place id

            $business_listing_url = strtok($downloadUrl, '?');
            $listing_id = (int)filter_var($business_listing_url, FILTER_SANITIZE_NUMBER_INT);

            $response = wp_remote_get($business_listing_url);

            if (is_array($response)) {
                $html_content = $response['body'];
            } else {
                wp_send_json_error([
                    'message' => __("Error finding key. Please contact with our plugin support team.", 'wpsocialreviews')
                ], 423);
            }

            $dom = new \DOMDocument();
            libxml_use_internal_errors(1);
            $dom->loadHTML($html_content);
            $xpath = new \DOMXpath($dom);

            $items = $xpath->query('//meta/@content');

            $key = '';
            $locale = '';

            $find_api_config = '"api_config":{';

            if ($items->length < 1) {
                wp_send_json_error([
                    'message' => __('Error 1: No key found.', 'wpsocialreviews')
                ], 423);
            } else {
                foreach ($items as $item) {
                    if (strpos($item->nodeValue, $find_api_config)) {
                        $data = json_decode($item->nodeValue, true);
                        $key = $data['api_config']['key'];
                        $locale = $data['locale'];
                        break;
                    }
                }
            }

            if ($key === "") {
                $find_api_config = '","api_config":';
                $position = strpos($html_content, $find_api_config);

                $api_locale_string = substr($html_content, $position - 20, 200);

                $find_api_config = 'api","key":"';
                $position = strpos($api_locale_string, $find_api_config);

                //find api key
                $tempendstring = substr($api_locale_string, $position, 100);
                $end = strpos($tempendstring, '"},');
                $key = substr($api_locale_string, $position + 12, $end - 12);

                //find locale
                $find_api_config = '"locale":"';
                $locale_pos = strpos($api_locale_string, $find_api_config);
                $locale = substr($api_locale_string, $locale_pos + 10, 2);
            }

            if ((!$key || empty($key)) && (!$listing_id || empty($listing_id))) {
                wp_send_json_error([
                    'message' => __('Error: Something went wrong. Please try again', 'wpsocialreviews')
                ], 423);
            }

            $limit = 5;
            $limit = apply_filters('wpsocialreviews/airbnb_reviews_limit_end_point', $limit);

            $offset = 0;

            $fetchInfoUrl = $this->remoteBaseUrl . '/listings/' . $listing_id . '?key=' . $key . '&_format=for_mobile_inbox';

            $experiences = strpos($downloadUrl, '/experiences/') !== false ? true : false;

            $fetchUrl = '';
            if ($experiences == 'true') {
                $fetchUrl = add_query_arg([
                    'key' => $key,
                    '_order' => 'language_country',
                    'reviewable_id' => $listing_id,
                    'reviewable_type' => 'MtTemplate',
                    'role' => 'guest',
                    '_limit' => $limit,
                    '_format' => 'for_p3'
                ], $this->remoteBaseUrl . '/reviews');
            } else {
                $fetchUrl = add_query_arg([
                    'key' => $key,
                    '_order' => 'language_country',
                    'listing_id' => $listing_id,
                    'role' => 'guest',
                    '_limit' => $limit,
                    '_format' => 'for_p3'
                ], $this->remoteBaseUrl . '/reviews');
            }

            $formated_business_info = false;

            $business_info = array();

            $fileurlcontents = $this->wpsr_file_get_contents_curl_browser($downloadUrl, '');//file_get_contents($downloadUrl);
            $html = wpsr_str_get_html($fileurlcontents);

            if ($experiences) {

                //find experience name
                $experienceName = '';
                if ($html->find('div._1r93ihzp')) {
                    $experienceName = $html->find('div._1r93ihzp', 0);
                }

                if (!$experienceName) {
                    if ($html->find('div._10ezbxd', 0)) {
                        if ($html->find('div._10ezbxd', 0)->find('h1._14i3z6h', 0)) {
                            if ($html->find('div._10ezbxd', 0)->find('h1._14i3z6h', 0)->plaintext) {
                                $experienceName = $html->find('div._10ezbxd', 0)->find('h1._14i3z6h', 0)->plaintext;
                            }
                        }
                    }
                }

                //find average rating
                $averageRating = '';
                if ($html->find('div._1r3fg0')) {
                    $averageRating = strip_tags($html->find('div._1r3fg0', 0));
                }

                if (!$averageRating) {
                    if ($html->find('div._1jf3c1i', 0)) {
                        if ($html->find('div._1jf3c1i', 0)->find('span[class=_6ijom0]', 0)) {
                            $averageRating = $html->find('div._1jf3c1i', 0)->find('span[class=_6ijom0]', 0);
                        }
                    }
                }

                //find total reviews
                $totalReviews = '';
                if ($html->find('span._yr6mxop')) {
                    $totalReviews = $html->find('span._yr6mxop', 0);
                }

                if (!$totalReviews) {
                    if ($html->find('span._pog3hg')) {
                        $totalReviews = strip_tags($html->find('span._pog3hg', 0));
                    }
                }

                if (!$totalReviews) {
                    if ($html->find('span._krjbj', 0)) {
                        $totalReviewsText = strip_tags($html->find('span._krjbj', 0));
                        $reviewsArray = explode(" ", $totalReviewsText);
                        $arrLen = count($reviewsArray);

                        if (!$totalReviews) {
                            $totalReviews = $reviewsArray[6];
                        }

                        if (!$averageRating) {
                            $averageRating = $reviewsArray[1];
                        }
                    }
                }

                $totalReviews = (int)filter_var(trim(strip_tags($totalReviews)), FILTER_SANITIZE_NUMBER_INT);

                $business_info['name'] = trim(strip_tags($experienceName));
                $business_info['star_rating'] = trim(strip_tags($averageRating));
                $business_info['reviews_count'] = $totalReviews;
                $formated_business_info = $this->getBusinessInfo($business_info);
            } else {
                $roomName = '';
                if ($html->find('h1._14i3z6h', 0)) {
                    $roomName = strip_tags($html->find('h1._14i3z6h', 0));
                }

                $averageReview = '';
                if ($html->find('span._1jpdmc0', 0)) {
                    $averageReview = strip_tags($html->find('span._1jpdmc0', 0));
                }

                $totalReviews = '';
                if ($html->find('span._1sqnphj', 0)) {
                    $totalReviewsText = strip_tags(trim($html->find('span._1sqnphj', 0)));
                    $totalReviews = str_replace(array('(', ')'), '', $totalReviewsText); // str_replace(array( '(', ')' ), '', $totalReviewsText);
                }

                if ($html->find('span._krjbj', 0)) {
                    $totalReviewsText = strip_tags($html->find('span._krjbj', 0));
                    $reviewsArray = explode(" ", $totalReviewsText);
                    $arrLen = count($reviewsArray);

                    if ($arrLen >= 6) {
                        if (!$totalReviews) {
                            $totalReviews = $reviewsArray[6];
                        }

                        if (!$averageReview) {
                            $averageReview = $reviewsArray[1];
                        }
                    }
                }

                $business_info['name'] = $roomName;
                $business_info['star_rating'] = $averageReview;
                $business_info['reviews_count'] = $totalReviews;

                $formated_business_info = $this->getBusinessInfo($business_info);

            }
            $response = wp_remote_get($fetchUrl);
            $data = json_decode(wp_remote_retrieve_body($response), true);
            //end: find airbnb reviews

            if (is_wp_error($data)) {
                throw new \Exception($data->get_error_message());
            }

            $oldSettings = $this->getApiSettings();

            if ($oldSettings['place_id'] != $listing_id) {
                $this->deletePlatformReviews();
            }

            if ($data['reviews']) {
                $this->saveApiSettings([
                    'api_key' => $key,
                    'place_id' => $listing_id,
                    'url_value' => $downloadUrl
                ]);
                //add cron event schedule here
                $this->doCronEvent();
                $this->syncRemoteReviews($data['reviews']);
                return $formated_business_info;
            } else {
                // clear cron event schedule
                // wp_clear_scheduled_hook( 'wpsr_daily_event_'.$this->platform );
                wp_send_json_error([
                    'message' => __('No reviews Found!', 'wpsocialreviews'),
                ], 423);
            }
        } else {
            wp_send_json_error([
                'message' => __('Please enter a valid url!', 'wpsocialreviews'),
            ], 423);
        }
    }

    public function formatData($review, $index)
    {
        $apiSettings = $this->getApiSettings();
        $reviewDate = $review['created_at'];
        return [
            'platform_name' => $this->platform,
            'source_id' => ArrayHelper::get($apiSettings, 'place_id'),
            'reviewer_name' => $review['reviewer']['first_name'],
            'review_title' => $this->platform . '_' . ($index + 1),
            'reviewer_url' => 'https://www.airbnb.com' . $review['reviewer']['profile_path'],
            'reviewer_img' => $review['reviewer']['picture_url'],
            'reviewer_text' => ArrayHelper::get($review, 'comments'),
            'rating' => ArrayHelper::get($review, 'rating'),
            'review_time' => date('Y-m-d H:i:s', strtotime($reviewDate)),
            'updated_at' => date('Y-m-d H:i:s'),
            'created_at' => date('Y-m-d H:i:s')
        ];
    }

    public function getBusinessInfo($data = array())
    {
        $info = get_option('social_review_airbnb_business_info');
        $businessInfo = [
            'name' => '',
            'url' => '',
            'address' => '',
            'average_rating' => '',
            'total_rating' => '',
            'phone' => '',
            'platform_name' => '',
            'status' => false
        ];
        if ($data && is_array($data)) {
            $businessInfo = [
                'name' => ArrayHelper::get($data, 'name'),
                'url' => $this->curUrl,
                'address' => '',
                'average_rating' => ArrayHelper::get($data, 'star_rating'),
                'total_rating' => ArrayHelper::get($data, 'reviews_count'),
                'phone' => '',
                'platform_name' => $this->platform,
                'status' => true
            ];
        }
        if ($info && is_array($info)) {
            $businessInfo = [
                'name' => ArrayHelper::get($info, 'name'),
                'url' => ArrayHelper::get($info, 'url'),
                'address' => ArrayHelper::get($info, 'address'),
                'average_rating' => ArrayHelper::get($info, 'average_rating'),
                'total_rating' => ArrayHelper::get($info, 'total_rating'),
                'phone' => ArrayHelper::get($info, 'phone'),
                'platform_name' => $this->platform,
                'status' => true
            ];
        }
        return $businessInfo;
    }

    public function saveApiSettings($settings)
    {
        return update_option($this->optionKey, $settings, 'no');
    }

    public function getApiSettings()
    {
        $settings = get_option($this->optionKey);
        if (!$settings || empty($settings['api_key'])) {
            $settings = [
                'api_key' => '',
                'place_id' => '',
                'url_value' => ''
            ];
        }
        return $settings;
    }

    public function getUserData()
    {

    }

    public function getAdditionalInfo()
    {

    }

    //for save settings like sync,
    public function getAdvanceSettings()
    {
        //if main settings is configured then do advance settings .. check url,api,place id exists
        $urlSettings = get_option($this->optionKey, false);
        $settings = false;
        if ($urlSettings || !empty($urlSettings['url_value'])) {
            $settings = get_option('airbnb_global_settings', false);
            if (!$settings) {
                $settings = get_option('wpsr_airbnb_global_settings', false);
            }
            if (!$settings) {
                $settings = array(
                    'global_settings' => array(
                        'auto_syncing' => 'false',
                        'fetch_review_recurrence' => 'daily'
                    )
                );
            }
        }
        wp_send_json_success([
            'message' => __('Settings Found Successfully', 'wpsocialreviews'),
            'settings' => $settings
        ], 200);
    }

    public function saveAdvanceSettings()
    {
        $args = wp_unslash($_REQUEST['args']);
        update_option('wpsr_airbnb_global_settings', $args);
        $this->doCronEvent();
        wp_send_json_success([
            'message' => __('Settings Saved Successfully', 'wpsocialreviews'),
        ], 200);
    }

    public function doCronEvent()
    {
        $settings = get_option('airbnb_global_settings', false);
        if (!$settings) {
            $settings = get_option('wpsr_airbnb_global_settings', false);
        }
        if ($settings) {
            //check sync is on or off, if off clear cron event
            $sync = isset($settings['global_settings']['auto_syncing']) ? $settings['global_settings']['auto_syncing'] : 'true';
            $recurrence = isset($settings['global_settings']['fetch_review_recurrence']) ? $settings['global_settings']['fetch_review_recurrence'] : 'daily';
            if ($sync === 'false') {
                // clear cron event
                wp_clear_scheduled_hook('wpsr_daily_event_' . $this->platform);
            } else if ($sync === 'true') {
                //clear the existing cron event
                wp_clear_scheduled_hook('wpsr_daily_event_' . $this->platform);
                wp_schedule_event(time(), $recurrence, 'wpsr_daily_event_' . $this->platform);
            }
        }
    }

}
